# Aneuploidy detection

Code created for bachelor thesis. The code involves snakemake pipeline with shell, Python, and R scripts. 

The pipeline was tested on Linux-based operating system on MetaCentrum server. 

## Source code

- Repository contains CSV files with precomputed ratios of diploid genome with coverage 1, so there is no need to simulate diploid genome straight-away, if not necessary. The CSV files are in directories `data/diploid/figures/zscore`. To generate new diploid genome with coverage 1×, just run the pipeline with scenario named `diploid` with parameters: `chrX 1 chrY 1` in the `scenario` section of a config file. 

## Installation
The only prerequisite is the installed Conda. For example, the Miniconda distribution available online (https://docs.conda.io/en/latest/miniconda.html#linux-installers). On Metacentrum, use 
        
        module add conda-modules

- Download the source code with additional data: 

        git clone \
        https://gitlab.fi.muni.cz/xguman/aneuploidy-detection.git
        cd aneuploidy-detection
    
- Setup the Conda virtual enviroment with requirements and name (default: `snake-aneu`) of enviroment stored in enviroment.yaml
    
        conda activate base
        conda env create --file enviroment.yaml
    
- Activate the enviroment
    
        conda activate snake-aneu

## Download Reference Genome

There should be a reference genome in FASTA format in a directory `data`. It is possible to use any reference genome, provided naming of sequecnes is consistent across the Snakemake pipeline, in the `scripts/change_ploidy.py` script, and in the config file. For the purpose of readability of the workflow, the following procedure of download stated in LD-PGTA repository on GitHub (\url{https://github.com/mccoy-lab/LD-PGTA/wiki/LD-PGTA-tutorial#seq_alignment}) was used:
```{r}
cd data
wget 'ftp://hgdownload.cse.ucsc.edu/goldenPath/hg38/chromosomes/chr?.fa.gz'
wget 'ftp://hgdownload.cse.ucsc.edu/goldenPath/hg38/chromosomes/chr??.fa.gz'
gunzip *.fa.gz
cat *.fa > hg38.fna
rm chr*.fa
samtools faidx hg38.fna
```

The `hg38.fa` dataset obtained by this procedure consists only of primary assembly chromosomes named "chrN" with index N, as was used in this thesis. 

It is important not to store reference genome in the same directory as other FASTA files generated through the workflow (`data/ref` in this case), as this can create cycles in the graph of jobs. Use `data` directory instead, or specify otherwise in a config file.  

In the last line of the previous commands, the `samtools faidx` was used to create `hg38.fna.fai` index file. 

## Config Files
Example:
````````````````````
scenarios:
  trisomy1: chr1 3
  diploid: chrX 1 chrY 1
  monosomy17: chr17 1
label:
  trisomy1: male (47,XY,+1)
  diploid: male (46,XY)
  monosomy17: male (45,XY,-17)
coverage:
  - 0.1
  - 0.5
short_read_length:
  150
reference_genome:
  "data/ref/hg38.fna"
resolution:
  x: 1000
  y: 400
````````````````````
**Description:** 

- "scenarios" can contains name of created scenario with argument as "chrN i chrM j", where N and M are the indices of chromosomes in a reference and i and j are their respective copy number in a output file. 

- "coverage" is the list of desired coverages to be generated from the scenarios

- "short read length" is the desired length of reads passed to the Wgsim simulator

- "reference genome" is the file that will be used as haploid reference

- "resolution" contains values of width (x) and height (y) in pixels. Plots resulted from R script will have the set size.

Config files should be stored in `configs` directory

## Running the Pipeline

- After the config is set up, it is possible to perform a "dry run" of a pipeline. This will only list jobs that will be performed:

```{r}    
        snakemake -np --configfile configs/config_name.yaml
```
- For a proper run of a pipeline, type:
```{r}
        snakemake --configfile configs/config_name.yaml --cores 6
```
    - "--cores" flag with an integer on input specifies the number of allocated processor units, thus number of parallel jobs that can be run at the same time. With higher number, there may be an unexpected halt of a pipeline, as the whole memory may be used, specifically when running multiple minimap2 aligning jobs. 
    
## Results
Every scenario generated has its own directory in `data`, e.g. `data/trisomy1`, `data/diploid`. Their respective sub-folders are directories for simulated reads (`reads`), aligned data (`aln`), sorted aligned data (`sorted`), and analysis results (`figures`). Apart from `figures` directory, every folder has `short` and `long` subfolders for storing data for their respective read lenghts. 
Folder `figures` consists of `zscore` and `ratios` subfolders. In `zscore` folder, there are plots for Z-score method, and CSV files with all the results of analysis, comparative method's copy number included. In `ratios` subfolder, there are just plots for comparative method of this thesis, without CSV files, as they are in full scale in `zscore` folder. 

Every file has a prescribed structure that can be put as: `short_<scenario>_<coverage>x.<file_extension>` for short-read file, and `long_<scenario>_<coverage>x.<file_extension>` for long-read files. `<coverage>` is a number without the decimal point. For example, the coverage 0.25× in file notation becomes "025". 



